
// ALL Environments share the same program counter and label!
var PC = 0;
var PCL = "__PC__";


class Environment{
  constructor(parent){
    this.__parent = (parent instanceof Environment) ? parent : null;
    this.__vars = {};
  }

  get PC(){return PC;}
  set PC(val){
    if (val < 0)
      throw new RangeError("Program Counter value is out of bounds.");
    PC = val;
  }

  get PCLabel(){return PCL;}
  set PCLabel(pcl){
    if (typeof(pcl) !== 'string' || pcl === "")
      throw new TypeError("Expected a non-zero-length string.");
    PCL = pcl;
  }

  hasVar(name){
    if (name === PCL || name in Object.keys(this.__vars))
      return true;
    if (this.__parent !== null)
      return this.__parent.hasVar(name);
    return false;
  }

  getVarValue(name){
    if (name === PCL){return PC;}
    if (name in Object.keys(this.__vars))
      return this.__vars[name];
    if (this.__parent !== null)
      return this.__parent.getVarValue(name);
    throw new Error("No var '" + name + "' defined.");
  }

  setVarValue(name, val){
    if (name === PCL){
      if (typeof(val) !== 'number' || val < 0)
        throw new Error("Special Variable '" + name + "' expected to be a possitive integer.");
      PC = Math.floor(val);
      return this;
    }
    if (!(name in Object.keys(this.__vars))){
      if (this.__parent.hasVar(name))
        return this.__parent.setVarVal(name, val);
    }
    this.__vars[name] = val;
    return this;
  }
}


module.exports = Environment;
