
const IO = require('../io.js');
const IMem = require('./imem.js');


class ROM extends IMem{
  constructor(pages){
    if (pages < 1)
      throw new RangeError("Memory requires positive page values.");
    super();
    this.__addr = 0;
    this.__map = new Uint8Array(pages * 256);
    // Allow writable events because RAM will inherit ROM and use the same
    // __io object.
    this.__io = new IO([], []);
  }

  get pages(){return this.__map.length / 256;}
  get size(){return this.__map.length;}

  get address(){return this.__addr;}
  set address(a){
    this.__addr = Math.min(this.__map.length - 1, Math.max(0, a));
  }

  get byte(){
    this.__io.triggerRead(this.__addr);
    return this.__map[this.__addr];
  }
  set byte(b){}


  onAddressRead(addr, fn){
    if (addr < 0 || addr >= this.size)
      throw new RangeError("Memory address is out of bounds."); 
    this.__io.onRead(addr, fn);
    return this;
  }


  peek(a){
    a = Math.min(this.__map.length - 1, Math.max(0, a));
    return this.__map[a];
  }


  load(address, data){
    let dc = data;
    if (address < 0 || address >= this.__map.length)
      throw new RangeError("Memory address out of range.");
    if (this.__map.length - address < dc.length)
      dc = dc.slice(0, this.__map.length - address);
    this.__map.set(dc, address);
    return dc.length;
  }


  clearPage(page){
    let addr = (page << 8) & 0xFF00;
    if (addr < 0 || addr >= this.__map.length)
      throw new RangeError("Memory address out of range.");
    this.__map.fill(0, addr, addr + 256);
    return this;
  }


  clear(){
    this.__map.fill(0);
    return this;
  }
}



module.exports = ROM;





